#!/usr/bin/perl

# Input a properly formatted SMTP message
# Output is same SMTP message sans certain headers or body items
#
# Options:
#  -a  Input is a bounced majordomo message (ie the headers we are
#      really interested in are actually in the message body).
#      Retains this format for further processing by, say, the
#      approve script.
#
#  -H  Don't strip nasty headers.
#
#  -B  Don't strip nasty body parts.
#
#  -F  Don't flatten out a multipart message (removes preamble and epilogue)
#
# These configurable regexps can take two forms:
#  /regexp/flags
#  s/search/replace/flags
# the latter is, obviously a search and replace. The search will always work,
# but the replace will just be null if -R was specified on the command line.
#
# One day these could be (optionally) be obtained from majodomo.cf...

$nastyheaders = <<'END';
/^Content\-Type\:\s+(?!text($|\/plain)).*/i
/^Content\-Transfer\-Encoding\: base64.*/i
/^X-MS-Attachment: /i
END

# The message body is crudely split into parts before processing each
# of these items.

# parts containing these headers will be zapped entirely
$nastyparts = <<'END';
/^Content\-Type\:\s+(?!text\/plain)/i
/^Content\-Transfer\-Encoding\: base64/i
END

# parts containing these items anywhere will have the items, plus
# any remaining lines, removed

$nastybody = <<'END';
/^<!DOCTYPE HTML PUBLIC "-\/\/W3C\/\/DTD W3 HTML\/\/EN">/i
/^<HTML>/i
/\bWINMAIL.DAT\b/i
/^begin \d{3} \S+/
/^\(This file must be converted with BinHex [\d\.]+\)/i
END


$tempdir = "sys\$scratch";


###### End of User Config ######


require "getopts.pl";

&Getopts("aHBF") || die("USAGE: mailstrip [-aFHB]\n");

$headexpr = &read_config($nastyheaders);
$partexpr = &read_config($nastyparts);
$bodyexpr = &read_config($nastybody);

&read_message(STDIN);


###### subs

sub read_config {

  local($expr) = "";

  foreach (split(/[\n\r]+/,$_[0])) {
    s/(^\s+|\s+$)//g;

    if ( m:^s/(.*[^\\])/(.*[^\\])/([eiosx]*)$: ) {
      $expr .= "s/$1/$2/gm$3 ".
               "&& (\$msg .= '  replaced $1 with $2\'.\"\n\");\n";
    }
    elsif ( m:^/(.*[^\\])/([iosx]*)$: ) {
      $expr .= "s/$1//gm$2 && (\$msg .= '  stripped $1\'.\"\n\");\n";
    }
    else { print STDERR "Ignoring : $_\n" }
  }
  $expr;
}

sub read_message {

  if ($opt_a) {
    $/ = '';
    my $null = <>;
    print $null;
    $/ = "\n";
  }

  &process_head();

  if ($opt_B) { print <> }
  elsif ($boundary) { &process_mime() }
  else { &process_body() }

}

sub process_head {

  local $hmsg = "";

  $/ = '';
  local $headers = <>;
  $/ = "\n";
  $headers =~ s/\n\s+/ /g;
  $headers =~ s/\s$/\n/g;

  if ($headers =~ /^Content\-Type\: multipart\/\w+\; boundary\=([^\n]+)/im) {
    $boundary = $1;
    $boundary =~ s/^([\s"']+)|([\s"']+$)//g;
    $oboundary = $boundary;
    $boundary =~ s/(\W)/\\$1/g;
  }

  unless ($opt_F) { 
    $headers =~ s/^Content\-Type\: multipart.*$/Content\-Type\: text\/plain/im
  }

  if ($opt_H) { print $headers,"\n" }
  else {
    local $msg = "";
    $_ = $headers;
    eval ($headexpr);
    s/\n\n+/\n/;
    if ($msg) { $hmsg .= $msg }
    print "$_\n";
  }

  print STDERR "In header:\n$hmsg" if $hmsg;

}

sub process_body {

  while (<>) {

    $msg = "";
    eval ($bodyexpr);
    if ($msg) {
      print STDERR "In part $part, skipping rest:\n$msg"; 
      return 0;
    }
    print  

  }

}


sub process_mime {

  local $bmsg = "";

  local $part = 1;
  local $skip = 0;
  local $bad = $opt_F?0:1;
  local $lasttype = "";
  open(OP, ">$tempdir:stripmail-$$-part.$part");

  unless ($opt_F) {
    print " === The original message was multipart MIME            ===\n",
          " === All non-text parts (attachments) have been removed ===\n\n";
  }
  while (<>) {

    if (/^\-\-$boundary(\-\-)?$/) {
      close(OP);
      if ($bad) {
        print STDERR "Skipped part $part:\n$bmsg" if $bmsg;
        print " === MIME part removed : ",($lasttype||"unknown type")," ===\n\n"
              unless ($part == 1);
      } else { 
        open(OP,"$tempdir:stripmail-$$-part.$part");
        print <OP>;
        close(OP);
      }
      unlink "$tempdir:stripmail-$$-part.$part";
      $skip = 0;
      $bad = 0;
      $bmsg = "";
      $part++;
      open(OP, ">$tempdir:stripmail-$$-part.$part");
      print OP if $opt_F;
      $_ = <>;
      if (/./) { # if no headers - it's text/plain or message/rfc822
        seek STDIN,0-length($_),1;
        $/ = '';
        $_ = <>;
        $/ = "\n";
      }
      s/\n\s+/ /g;
      s/\s$/\n/g;
      ($lasttype) = /^Content-Type:\s+(\S+)/mi;
      $msg = "";
      eval ($partexpr);
      if ($msg) {
        print STDERR "Header in part $part, zapping whole part\n$msg";
        $bad++; 
        $skip++;
      }
      elsif ($opt_F) { print OP "$_\n" }
      next;
    }
    
    next if $skip;

    $msg = "";
    eval ($bodyexpr);
    if ($msg) {
      print STDERR "In part $part, skipping rest:\n$msg"; 
      $skip++ ; next
    }
    print OP  
  }

  close(OP);
  unless ($bad) {
    open(OP,"$tempdir:stripmail-$$-part.$part");
    print <OP>;
    close(OP);
  }
  unlink "$tempdir:stripmail-$$-part.$part";

}
